varying vec4 Position;
varying vec3 Normal;
varying vec2 TexCoords;

uniform sampler2D TextureID;
uniform sampler2D NormalsID;

vec4 shading(in vec4 Position)
{
	// Ambient component.
	vec4 Color = gl_LightSource[0].ambient;
	vec4 base = texture(TextureID,TexCoords);
	vec3 normalPt = texture(NormalsID,TexCoords).rbg;
	
	if(normalPt.x != 0 && normalPt.y != 0 && normalPt.z != 0)
	{
		Normal = gl_NormalMatrix * (2.0 * normalPt - 1.0);
	}
	Normal = normalize(Normal);
	
	if(gl_FrontMaterial.ambient.w != 1.0)
	{
		Color += gl_FrontMaterial.ambient * gl_FrontMaterial.ambient.w;
	}

	// Light vector in View space.
	vec3 LightVector = normalize((gl_LightSource[0].position - Position).xyz);

	float DiffuseFactor = dot(Normal, LightVector);
	if (DiffuseFactor>0.0)
	{
		// Diffuse component.
		Color += base + base*gl_LightSource[0].diffuse * DiffuseFactor;
		
		// Reflected light vector in View space.
		vec3 ReflectedLightVector = reflect(-LightVector, Normal);
        
		// View vector in View space.
		vec3 ViewVector = normalize(-Position.xyz);

		float SpecularFactor = dot(ViewVector, ReflectedLightVector);
		if (SpecularFactor>0.0)
		{
			// Specular component.
			Color += vec4(1, 1, 1, 1) * gl_LightSource[0].specular * pow(SpecularFactor, gl_LightSource[0].spotExponent);
		}
	}
	return Color;
}

void main() 
{
	// Output color.
	gl_FragColor = shading(Position);
}

